package net.sf.sido.spring;

import net.sf.sido.DataFactory;
import net.sf.sido.pojo.DOFactory;
import net.sf.sido.support.DataFactories;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;

import javax.annotation.PostConstruct;
import java.util.Map;

/**
 * Loads the {@link net.sf.sido.pojo.DOFactory} and sets it as a default. If no DOFactory
 * is defined in the context, it will create one according to the given parameters. If no DataFactory
 * is provided to create this DOFactory, a {@linkplain net.sf.sido.support.DataFactories#typedFactory() typed factory}
 * is created.
 */
public class DOFactoryLoader {

    private final ApplicationContext applicationContext;

    private DOFactory defaultDOFactory;
    private DataFactory defaultDataFactory;

    @Autowired
    public DOFactoryLoader(ApplicationContext applicationContext) {
        this.applicationContext = applicationContext;
    }

    @PostConstruct
    public void load() {
        // Gets the DOFactory
        DOFactory doFactory = getDOFactory();
        // Sets it as default
        DOFactory.setDefaultFactory(doFactory);
    }

    protected DOFactory getDOFactory() {
        // Default one?
        if (defaultDOFactory != null) {
            return defaultDOFactory;
        }
        // Gets it from the context first
        DOFactory doFactory = getByType(DOFactory.class);
        // Found
        if (doFactory != null) {
            return doFactory;
        }
        // Creates it
        DataFactory dataFactory = getDataFactory();
        doFactory = new DOFactory(dataFactory, true);
        // OK
        return doFactory;
    }

    protected DataFactory getDataFactory() {
        // Default one?
        if (defaultDataFactory != null) {
            return defaultDataFactory;
        }
        // Gets it from the context first
        DataFactory dataFactory = getByType(DataFactory.class);
        // Found
        if (dataFactory != null) {
            return dataFactory;
        }
        // Creates it
        return createDataFactory();
    }

    protected DataFactory createDataFactory() {
        return DataFactories.typedFactory();
    }

    protected <T> T getByType(Class<T> type) {
        Map<String, T> map = applicationContext.getBeansOfType(type);
        if (map != null && !map.isEmpty()) {
            if (map.size() > 1) {
                throw new IllegalStateException(String.format("More than 1 bean of type %s was found in the context: %d", type.getName(), map.size()));
            } else {
                return map.values().iterator().next();
            }
        } else {
            return null;
        }
    }

    public DOFactory getDefaultDOFactory() {
        return defaultDOFactory;
    }

    public void setDefaultDOFactory(DOFactory defaultDOFactory) {
        this.defaultDOFactory = defaultDOFactory;
    }

    public DataFactory getDefaultDataFactory() {
        return defaultDataFactory;
    }

    public void setDefaultDataFactory(DataFactory defaultDataFactory) {
        this.defaultDataFactory = defaultDataFactory;
    }
}
